@@ -16,7 +16,6 @@
  *  You should have received a copy of the GNU General Public License
  *  along with Mtools.  If not, see <http://www.gnu.org/licenses/>.
  */
-#include "msdos.h"
 #include "llong.h"
 typedef struct dos_name_t dos_name_t;
 
@@ -95,12 +94,8 @@ FILE *opentty(int mode);
 
 int is_dir(Stream_t *Dir, char *path);
 
-int dir_grow(Stream_t *Dir, int size);
-int match(const wchar_t *, const wchar_t *, wchar_t *, int,  int);
+int dir_grow(Stream_t *Dir, unsigned int size);
 
-wchar_t *unix_name(doscp_t *fromDos,
-		   const char *base, const char *ext, uint8_t Case,
-		   wchar_t *answer);
 void *safe_malloc(size_t size);
 Stream_t *open_dos2unix(Stream_t *Next,int convertCharset);
 Stream_t *open_unix2dos(Stream_t *Next,int convertCharset);
@@ -124,32 +119,22 @@ if(source)target=source
 
 #define compare(ref,testee) ((ref) && (ref) != (testee))
 
-UNUSED(static __inline__ char ch_toupper(char ch))
+static inline char ch_toupper(char ch)
 {
         return (char) toupper( (unsigned char) ch);
 }
 
-UNUSED(static __inline__ char ch_tolower(char ch))
+static inline char ch_tolower(char ch)
 {
         return (char) tolower( (unsigned char) ch);
 }
 
-UNUSED(static __inline__ wchar_t ch_towupper(wchar_t ch))
-{
-        return (wchar_t) towupper( (wint_t) ch);
-}
-
-UNUSED(static __inline__ wchar_t ch_towlower(wchar_t ch))
-{
-        return (wchar_t) towlower( (wint_t) ch);
-}
-
-UNUSED(static __inline__ void init_random(void))
+static inline void init_random(void)
 {
 	srandom((unsigned int)time (0));
 }
 
-UNUSED(static __inline__ size_t ptrdiff (const char *end, const char *begin))
+static inline size_t ptrdiff (const char *end, const char *begin)
 {
 	return (size_t) (end-begin);
 }
@@ -182,9 +167,8 @@ char get_default_drive(void);
 void set_cmd_line_image(char *img);
 void check_number_parse_errno(char c, const char *optarg, char *endptr);
 void read_config(void);
-off_t str_to_offset_with_end(const char *str, char **endp);
 mt_off_t str_to_off_with_end(const char *str, char **endp);
-off_t str_to_offset(char *str);
+mt_off_t str_to_offset(char *str);
 uint32_t parseSize(char *sizeStr);
 unsigned int strtoui(const char *nptr, char **endptr, int base);
 unsigned int atoui(const char *nptr);
@@ -264,7 +248,7 @@ FILE *open_mcwd(const char *mode);
 void unlink_mcwd(void);
 
 #ifndef OS_mingw32msvc
-ssize_t safePopenOut(const char **command, char *output, size_t len);
+ssize_t safePopenOut(const char* const* command, char *output, size_t len);
 #endif
 
 #define ROUND_DOWN(value, grain) ((value) - (value) % (grain))
@@ -274,4 +258,14 @@ ssize_t safePopenOut(const char **command, char *output, size_t len);
 #define O_BINARY 0
 #endif
 
+extern const char *mversion;
+extern const char *mdate;
+extern const char *mformat_banner;
+
+extern char *Version;
+extern char *Date;
+
+
+int init(char drive, int mode);
+
 #endif
