@@ -18,10 +18,10 @@
 #include "sysincludes.h"
 #include "mtools.h"
 
-const char *mversion="4.0.43";
+const char *mversion="4.0.45";
 
 /* Multiple releases on same day should be marked with (b), (cd), (d) after
  * date string below */
-const char *mdate = "March 21st, 2023";
+const char *mdate = "September 28th, 2024";
 
-const char *mformat_banner = "MTOO4043";
+const char *mformat_banner = "MTOO4045";
